# ==============================================================================
# Copyright [2013] [Kevin Carter]
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
__author__ = "Kevin Carter"
__contact__ = "Kevin Carter"
__email__ = "kevin.carter@rackspace.com"
__copyright__ = "2013 All Rights Reserved"
__date__ = "2013-23-03"
__version__ = "0.003"
__status__ = "Seems to Work"
__appname__ = "xenosimager"
__license__ = "Apache2.0"
__description__ = ("Create an image of a instance from data obtained from"
                   " within the instance")
__urlinformation__ = "http://github.com/cloudnull/transporter"

# The Version Of the Application
V_N = '%s' % __version__

# The Version and Information for the application
VINFO = ('%(version)s, developed by %(author)s, '
         'FYI : %(copyright)s' % {'version': __version__,
                                  'author': __author__,
                                  'copyright': __copyright__})

